@@ -11,6 +11,7 @@ __all__ = ["Queue"]
 
 
 class Queue(OrchestratorHTTP):
+    classes = ["Comments", "Status", "Reference"]
     """
     Constructor. 
 
@@ -55,13 +56,14 @@ class Queue(OrchestratorHTTP):
         data = self._get(url)
         return data
 
-    def start(self, machine_identifier, specific_content=None, reference=None):
+    def start(self, machine_identifier, specific_content=None, reference=None, fields=None):
         """
             Starts a given transaction
 
             @machine_identifier: the machine's unique identifier
             @specific_content: the specific content of the transaction
             @reference: a reference from the specific content 
+            @fields: a dictionary of additional fields to be added to the specific content
         """
         ran_uuid = str(uuid4())
         batch_id = str(uuid.uuid4())
@@ -74,12 +76,19 @@ class Queue(OrchestratorHTTP):
                 "SpecificContent": specific_content,
             }
         }
-
         if reference:
-            format_body_start["transactionData"]["Reference"] = f"{reference}#{batch_id}"
-            format_body_start["transactionData"]["SpecificContent"]["ItemID"] = reference
+            try:
+                value = format_body_start["transactionData"]["SpecificContent"][reference]
+                format_body_start["transactionData"]["Reference"] = f"{value}#{batch_id}"
+            except KeyError as err:
+                if reference in err.args:
+                    logging.error(f"Invalid reference: {reference} not found in SpecificContent")
+                    raise Exception(f"Invalid reference: {reference} not found in SpecificContent")
+            format_body_start["transactionData"]["SpecificContent"]["ItemID"] = value
             format_body_start["transactionData"]["SpecificContent"]["ReferenceID"] = ran_uuid
             format_body_start["transactionData"]["SpecificContent"]["BatchID"] = batch_id
+        if fields:
+            format_body_start["transactionData"]["SpecificContent"].update(fields)
 
         url = f"{self.base_url}{endpoint}"
         return self._post(url, body=format_body_start)
@@ -124,7 +133,11 @@ class Queue(OrchestratorHTTP):
             @returns: a list of QueueItem objects of the given queue (Maximum number of results: 1000)
         """
         endpoint = "/QueueItems"
-        odata_filter = {"$Filter": f"QueueDefinitionId eq {self.id}"}
+        if "$filter" in options:
+            # print("true")
+            odata_filter = {"$filter": f"QueueDefinitionId eq {self.id} and {options['$filter']}"}
+        else:
+            odata_filter = {"$Filter": f"QueueDefinitionId eq {self.id}"}
         if options:
             odata_filter.update(options)
         query_params = urlencode(odata_filter)
@@ -261,3 +274,48 @@ class Queue(OrchestratorHTTP):
         endpoint = f"/QueueDefinitions({self.id})"
         url = f"{self.base_url}{endpoint}"
         return self._delete(url)
+
+    def get_queue_item_comments(self, q=None):
+        endpoint = "/QueueItemComments"
+        if q:
+            query_params = urlencode({
+                "$filter": q
+            })
+            url = f"{self.base_url}{endpoint}?{query_params}"
+        url = f"{self.base_url}{endpoint}"
+        return self._get(url)
+
+    def check_duplicate(self, endpoint, value):
+        """
+        Checks if there is an attribute cls 
+        with the value value in the queue. 
+        This checks all items of the queue with the attribute and value
+        indicated
+
+            - `param` cls: possible values are "Comment", "Status", "Reference"
+
+        Example usage: 
+
+        ```py
+        client = Orchestrator()
+        queue = client.get_queue_by_id(188797)
+
+        # checks among all comments in all items, and returns true
+        # if it finds one item with that item comment
+        bool = queue.check_duplicate("Comments", "hello world")
+        ```
+        """
+        if endpoint not in self.classes:
+            raise NotImplementedError
+        elif endpoint == "Comments":
+            comments = self.get_queue_item_comments()["value"]
+            # print(comments)
+            for comment in comments:
+                # print(comment["Text"])
+                if comment["Text"] == value:
+                    return True
+        elif endpoint == "Status":
+            pass
+        elif endpoint == "Reference":
+            item = self.get_queue_items(options={"$filter": f"contains(Reference,'{value}')"})
+            return item
